@@ -29,15 +29,18 @@ try:
 except(ImportError):
     pass
 
+import sys
 import types
 __all__ = [f for f in dir() if f[0] != '_' and not isinstance(eval(f),
            types.ModuleType)]
 
-del f  #XXX not necessary with Python3
+if sys.version_info.major == 2:
+    del f
+del sys
 del types
 
 I = IdentityOperator()
 O = ZeroOperator()
 X = Variable('X')
 
-__version__ = '0.12.14'
+__version__ = u'0.13'
