@@ -8,8 +8,9 @@ try:
 except:
     pass
 import scipy.sparse as sp
+import scipy.sparse.sparsetools as sps
 import sys
-from itertools import izip
+
 from scipy.sparse.linalg import eigsh
 from .core import (
     BlockRowOperator, BroadcastingBase, CompositionOperator, ConstantOperator,
@@ -324,25 +325,11 @@ class DenseOperator(DenseBlockDiagonalOperator):
 @contiguous
 @update_output
 class SparseBase(Operator):
-    def __init__(self, matrix, dtype=None, shapein=None, shapeout=None,
-                 **keywords):
+    def __init__(self, matrix, dtype=None, **keywords):
         if dtype is None:
             dtype = matrix.dtype
-        if shapein is None:
-            shapein = matrix.shape[1]
-        elif product(shapein) != matrix.shape[1]:
-            raise ValueError(
-                "The input shape '{0}' is incompatible with the sparse matrix "
-                "shape {1}.".format(shapein, matrix.shape))
-        if shapeout is None:
-            shapeout = matrix.shape[0]
-        elif product(shapeout) != matrix.shape[0]:
-            raise ValueError(
-                "The output shape '{0}' is incompatible with the sparse matrix"
-                " shape {1}.".format(shapeout, matrix.shape))
         self.matrix = matrix
-        Operator.__init__(self, shapein=shapein, shapeout=shapeout,
-                          dtype=dtype, **keywords)
+        Operator.__init__(self, dtype=dtype, **keywords)
 
     @property
     def nbytes(self):
@@ -358,7 +345,7 @@ class SparseBase(Operator):
         if isinstance(m, sp.dok_matrix):
             sizeoftuple = sys.getsizeof(())
             return (24 * m.ndim + m.dtype.itemsize +
-                    2 * sizeoftuple + 24) * len(m.viewitems())
+                    2 * sizeoftuple + 24) * len(m.items())
         try:
             return m.data.nbytes
         except AttributeError:
@@ -406,6 +393,18 @@ class SparseOperator(SparseBase):
         if isinstance(matrix, sp.lil_matrix):
             raise TypeError('The LIL format is not suited for arithmetic opera'
                             'tions.')
+        if shapein is None:
+            shapein = matrix.shape[1]
+        elif product(shapein) != matrix.shape[1]:
+            raise ValueError(
+                "The input shape '{0}' is incompatible with the sparse matrix "
+                "shape {1}.".format(shapein, matrix.shape))
+        if shapeout is None:
+            shapeout = matrix.shape[0]
+        elif product(shapeout) != matrix.shape[0]:
+            raise ValueError(
+                "The output shape '{0}' is incompatible with the sparse matrix"
+                " shape {1}.".format(shapeout, matrix.shape))
         SparseBase.__init__(self, matrix, dtype=dtype, shapein=shapein,
                             shapeout=shapeout, **keywords)
         self.set_rule('T', lambda s: SparseOperator(s.matrix.transpose()))
@@ -421,11 +420,11 @@ class SparseOperator(SparseBase):
             raise ValueError('Invalid reduction operation.')
         m = self.matrix
         if isinstance(m, sp.dok_matrix):
-            for (i, j), v in m.iteritems():
+            for (i, j), v in m.items():
                 output[i] += v * input[j]
             return
         M, N = m.shape
-        fn = getattr(sp.sparsetools, m.format + '_matvec')
+        fn = getattr(sps, m.format + '_matvec')
         if isinstance(m, (sp.csr_matrix, sp.csc_matrix)):
             fn(M, N, m.indptr, m.indices, m.data, input, output)
         elif isinstance(m, sp.coo_matrix):
@@ -1208,10 +1207,10 @@ class BandOperator(Operator):
         # diag
         out[:] = self.ab[self.ku] * x
         # upper part
-        for i in xrange(self.ku):
+        for i in range(self.ku):
             j = self.ku - i
             out[:-j] += self.ab[i, j:] * x[j:]
-        for i in xrange(self.ku, self.kl + self.ku):
+        for i in range(self.ku, self.kl + self.ku):
             # lower part
             out[i:] += self.ab[i + 1, :-i] * x[:-i]
 
@@ -1221,10 +1220,10 @@ class BandOperator(Operator):
         # diag
         out = self.rab[self.ku] * x
         # upper part
-        for i in xrange(rku):
+        for i in range(rku):
             j = rku - i
             out[:-j] += rab[i, j:] * x[j:]
-        for i in xrange(rku, rkl + rku):
+        for i in range(rku, rkl + rku):
             # lower part
             out[i:] += rab[i + 1, :-i] * x[:-i]
 
@@ -1248,7 +1247,7 @@ class BandOperator(Operator):
         kl, ku = self.kl, self.ku
         rku, rkl = kl, ku
         rab = np.zeros(ab.shape, dtype=ab.dtype)
-        for i in xrange(- kl, ku + 1):
+        for i in range(- kl, ku + 1):
             rab[_band_diag(rku, -i)] = self.diag(i)
         return rab
 
@@ -1288,6 +1287,7 @@ class UpperTriangularOperator(BandOperator):
         BandOperator.__init__(self, ab, kl, ku, **kwargs)
 
 
+@square
 @symmetric
 class SymmetricBandOperator(Operator):
     """
@@ -1308,7 +1308,7 @@ class SymmetricBandOperator(Operator):
 
     def direct(self, x, out):
         out[:] = self.ab[0] * x
-        for i in xrange(1, self.ab.shape[0]):
+        for i in range(1, self.ab.shape[0]):
             # upper part
             out[:-i] += self.ab[i, :-i] * x[i:]
             # lower part
@@ -1362,6 +1362,7 @@ class SymmetricBandOperator(Operator):
 
 @real
 @linear
+@square
 @symmetric
 @inplace
 class SymmetricBandToeplitzOperator(Operator):
@@ -1451,7 +1452,7 @@ class SymmetricBandToeplitzOperator(Operator):
                     dtype):
         firstrow = firstrow.reshape((-1, ncorr + 1))
         kernel = empty((firstrow.shape[0], fftsize // 2 + 1), dtype)
-        for f, k in izip(firstrow, kernel):
+        for f, k in zip(firstrow, kernel):
             rbuffer[:ncorr+1] = f
             rbuffer[ncorr+1:-ncorr] = 0
             rbuffer[-ncorr:] = f[:0:-1]
@@ -1494,6 +1495,7 @@ class DifferenceOperator(Operator):
         return tuple(shape)
 
 
+@square
 @symmetric
 class EigendecompositionOperator(CompositionOperator):
     """
